/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package core;

import java.util.ResourceBundle;



/**
 * Class that contains all the filters used by
 * FileDialogs to open or to save files
 *
 */
public class FileDialogFilter {

	/**
	 * messages that are to be used by the entire GUI
	 */
	private static ResourceBundle messages = Loader.getMessages();

	/**
	 * Method that get's the project filter name from locale
	 * @return a string with the filter name
	 */
	public static String projectName() {
		return messages.getString("FILE_DIALOG_TITLE-PROJECT_FILTER");
	}

	/**
	 * Method that gets the project filter extension
	 * @return a string with the filter extension
	 */
	public static String projectExtension() {
		return "*.proj;";
	}

	/**
	 * Method that get's the jpeg filter name from locale
	 * @return a string with the filter name
	 */
	public static String jpegName() {
		return messages.getString("FILE_DIALOG_TITLE-JPEG_FILTER");
	}

	/**
	 * Method that get's the jpeg filter extensions
	 * @return a string with the filter extensions
	 */
	public static String jpegExtension() {
		return "*.jpg;*.jpeg;*.jpe;*.JPG;*.JPEG;*.JPE;";
	}

	/**
	 * Method that get's the png filter name from locale
	 * @return a string with the filter name
	 */
	public static String pngName() {
		return messages.getString("FILE_DIALOG_TITLE-PNG_FILTER");
	}

	/**
	 * Method that get's the png filter extension
	 * @return a string with the filter extension
	 */
	public static String pngExtension() {
		return "*.png;*.PNG;";
	}

	/**
	 * Method that get's the All Images filter name from locale
	 * @return a string with the filter name
	 */
	public static String AllImagesName() {
		return messages.getString("FILE_DIALOG_TITLE-ALLIMAGES_FILTER");
	}

	/**
	 * Method that get's the All Images filter extensions
	 * @return a string with the filter extensions
	 */
	public static String AllImagesExtension() {
		return jpegExtension() + pngExtension();
	}
}