/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package tabs;

import java.util.ArrayList;
import java.util.ResourceBundle;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Scale;

import core.Loader;

/**
 * Instances of this class provide methods that
 * can be used to help a tab drawing it's GUI 
 *
 */
public class TabGuiUtility {

	/**
	 * The 3 main composites on this tab
	 */
	private Composite upperComposite = null;
	private Composite colorsComposite = null;
	private Composite bottomComposite = null;

	/**
	 * Composite used to hold {@link ColorButton}'s
	 */
	private Composite coloredButtonsComposite = null;

	/**
	 * Width and height for the color buttons
	 */
	private final int COLOR_BUTTON_WIDTH = 22;
	private final int COLOR_BUTTON_HEIGHT = 26;	

	/**
	 * Used to store the temporary selected button in the colorsComposite
	 */
	private ColorButton selectedButton = null;

	/**
	 * The reference to the first colored button created so that it can be selected by default
	 */
	private ColorButton firstButton = null;

	/**
	 * Separator labels
	 */
	private Label labelSeparatorUp = null;
	private Label labelSeparatorDown = null;
	private Label labelSeparator = null;

	/**
	 * The colorDifference widget
	 */
	private ColorDifference colorDifference = null;

	/**
	 * Label used to display the tab's title
	 */
	private Label labelTabTitle = null;

	//Scale for the size and opacity
	private Scale sizeScale = null;
	private Scale opacityScale = null;

	//labels for the size and opacity scale
	private Label labelSizeScale = null;
	private Label labelOpacityScale = null;

	/**
	 * The composite used to add all components
	 */
	private final Composite mainComposite;

	/**
	 * messages that are to be used by the entire GUI
	 */
	private ResourceBundle messages;

	/**
	 * Creates a new instance of the current class
	 * given the composite used for drawings
	 * @param composite
	 */
	public TabGuiUtility(Composite composite) {
		this.mainComposite = composite;

		messages = Loader.getMessages();

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		composite.setLayout(gridLayout);

	}

	/**
	 * Create the upper composite of a tab
	 * which contains the tab's title
	 * @param labelText tab's title
	 */
	public void createUpperComposite(String labelText) {

		//grid data for the title label
		GridData gridDataLabelTitle = new GridData();
		gridDataLabelTitle.grabExcessHorizontalSpace = true;

		//grid data for the separator label
		GridData gridDataLabelSeparator = new GridData();
		gridDataLabelSeparator.grabExcessHorizontalSpace = true;
		gridDataLabelSeparator.horizontalSpan = 2;
		gridDataLabelSeparator.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid data for the composite
		GridData gridDataComposite = new GridData();
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//initialize the composite
		upperComposite = new Composite(mainComposite, SWT.NONE);
		upperComposite.setLayout(new GridLayout());
		upperComposite.setLayoutData(gridDataComposite);

		//initialize the label
		labelTabTitle = new Label(upperComposite, SWT.NONE);
		labelTabTitle.setLayoutData(gridDataLabelTitle);
		labelTabTitle.setText(labelText);

		//initialize the separator label
		labelSeparator  = new Label(upperComposite, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelSeparator.setLayoutData(gridDataLabelSeparator);

	}

	/**
	 * Create the color buttons composite
	 * @param colorsList the list of colors to create the color buttons (one for each color)
	 * @return the new created list of {@link ColorButton}'s
	 */
	public ArrayList<ColorButton> createColorButtonsComposite(Color[] colorsList) {

		//grid data for the composite
		GridData gridDataComposite = new GridData();
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.CENTER;

		//grid data for the color difference
		GridData gridDataColorDifference = new GridData();
		gridDataColorDifference.heightHint = COLOR_BUTTON_HEIGHT;
		gridDataColorDifference.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid layout for the colored buttons composite
		GridLayout gridLayoutColoredButtonsComposite = new GridLayout();
		gridLayoutColoredButtonsComposite.numColumns = 6;
		gridLayoutColoredButtonsComposite.marginHeight = 0;
		gridLayoutColoredButtonsComposite.marginWidth = 0;

		//initialize the composite
		colorsComposite = new Composite(mainComposite, SWT.NONE);
		colorsComposite.setLayout(new GridLayout());
		colorsComposite.setLayoutData(gridDataComposite);

		//initialize the color difference
		colorDifference = new ColorDifference(colorsComposite, SWT.NONE);
		colorDifference.setLayoutData(gridDataColorDifference);

		//initialize the colored buttons composite
		coloredButtonsComposite = new Composite(colorsComposite, SWT.NONE);
		coloredButtonsComposite.setLayout(gridLayoutColoredButtonsComposite);

		//create the colored buttons
		ArrayList<ColorButton> coloredButtonList = createColorButtons(colorsList);

		//by default select the first colored button
		colorDifference.setCurrentColor(firstButton.getBackground());
		colorDifference.setNewColor(firstButton.getBackground());
		selectedButton = firstButton;
		selectedButton.setSelected(true);
		return coloredButtonList;

	}

	/**
	 * This method initializes colorButtons	
	 * @param colorsList the list of colors (one {@link ColorButton} will be created for every given {@link Color})
	 */
	private ArrayList<ColorButton> createColorButtons(Color[] colorsList) {

		ArrayList<ColorButton> coloredButtonsList = new ArrayList<ColorButton>();

		//for each color in colorList create a ColorButton
		for (int i = 0; i < colorsList.length; i++) {

			//create a new colored button and add it to the list
			final ColorButton cb = new ColorButton(coloredButtonsComposite, SWT.NONE);
			cb.setBackground(colorsList[i]);
			coloredButtonsList.add(cb);

			//set the mouse enter event
			cb.addMouseTrackListener(new org.eclipse.swt.events.MouseTrackAdapter() {
				public void mouseEnter(org.eclipse.swt.events.MouseEvent e) {
					colorDifference.setNewColor(cb.getBackground());
				}
			});

			//set the mouse up event
			cb.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
				public void mouseUp(org.eclipse.swt.events.MouseEvent e) {
					//select the clicked ColorButton
					if (selectedButton != null) {
						selectedButton.setSelected(false);						
					}
					selectedButton = cb;
					selectedButton.setSelected(true);
					colorDifference.setCurrentColor(cb.getBackground());
				}
			});

			//set the mouse exit event
			cb.addMouseTrackListener(new org.eclipse.swt.events.MouseTrackAdapter() {
				public void mouseExit(org.eclipse.swt.events.MouseEvent e) {
					//TODO maybe this listener should be on other control than colorButton
					colorDifference.setNewColor(colorDifference.getCurrentColor());
				}
			});

			//TODO code for TAB pressed and move between colors with arrow keys missing

			//set colored button size
			GridData gridData = new GridData();
			gridData.widthHint = COLOR_BUTTON_WIDTH;
			gridData.heightHint = COLOR_BUTTON_HEIGHT;
			cb.setLayoutData(gridData);

			//retain the first button reference
			if (i == 0) { firstButton = cb; }

		}

		return coloredButtonsList;

	}

	/**
	 * Returns the current selected color in AWT format
	 * @return the current selected color in AWT format
	 */
	public java.awt.Color getCurrentColor() {
		if (selectedButton == null) {
			return null;
		} else {
			return colorDifference.getAwtCurrentColor();
		}
	}

	/**
	 * This method initializes bottomComposite	
	 */
	public void createBottomComposite() {

		//grid data for the separator label up
		GridData gridDataSeparatorLabelUp = new GridData();
		gridDataSeparatorLabelUp.grabExcessHorizontalSpace = true;
		gridDataSeparatorLabelUp.horizontalSpan = 2;
		gridDataSeparatorLabelUp.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid data for the separator label down
		GridData gridDataSeparetorLabelDown = new GridData();
		gridDataSeparetorLabelDown.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridDataSeparetorLabelDown.horizontalSpan = 2;
		gridDataSeparetorLabelDown.grabExcessHorizontalSpace = true;

		//grid data for the opacity scale
		GridData gridDataOpacityScale = new GridData();
		gridDataOpacityScale.grabExcessHorizontalSpace = true;
		gridDataOpacityScale.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid data for the size scale
		GridData gridDataSizeScale = new GridData();
		gridDataSizeScale.grabExcessHorizontalSpace = true;
		gridDataSizeScale.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid data for the composite
		GridData gridDataComposite = new GridData();
		gridDataComposite.verticalAlignment = org.eclipse.swt.layout.GridData.END;
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridDataComposite.grabExcessVerticalSpace = true;

		//grid layout for the composite
		GridLayout gridLayoutComposite = new GridLayout();
		gridLayoutComposite.numColumns = 2;

		//initialize the composite
		bottomComposite = new Composite(mainComposite, SWT.NONE);
		bottomComposite.setLayout(gridLayoutComposite);
		bottomComposite.setLayoutData(gridDataComposite);

		//initialize the upper label separator
		labelSeparatorUp = new Label(bottomComposite, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelSeparatorUp.setLayoutData(gridDataSeparatorLabelUp);

		//initialize the size scale label
		labelSizeScale = new Label(bottomComposite, SWT.NONE);
		labelSizeScale.setText(messages.getString("TAB_GUI_UTILITY_AREA"));

		//initialize the size scale
		sizeScale = new Scale(bottomComposite, SWT.NONE);
		sizeScale.setSelection(50);
		sizeScale.setLayoutData(gridDataSizeScale);

		//initialize the opacity scale label
		labelOpacityScale = new Label(bottomComposite, SWT.NONE);
		labelOpacityScale.setText(messages.getString("TAB_GUI_UTILITY_OPACITY"));

		//initialize the opacity scale
		opacityScale = new Scale(bottomComposite, SWT.NONE);
		opacityScale.setSelection(50);
		opacityScale.setLayoutData(gridDataOpacityScale);

		//initialize the lower separator label
		labelSeparatorDown = new Label(bottomComposite, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelSeparatorDown.setLayoutData(gridDataSeparetorLabelDown);

	}

	/**
	 * Returns the current size scale value
	 * @return the current size scale value
	 */
	public Scale getStrokeScale() {
		return sizeScale;
	}

	/**
	 * Returns the current opacity scale value
	 * @return the current opacity scale value
	 */
	public Scale getAlphaScale() {
		return opacityScale;
	}

	/**
	 * Select a given colored button
	 * @param cb the {@link ColorButton} to be selected
	 */
	public void setSelectedColorButton(ColorButton cb) {
		if (cb == null) {
			if (selectedButton != null) {
				selectedButton.setSelected(false);
				selectedButton = null;
			}
		} else {
			if (selectedButton != null) {
				selectedButton.setSelected(false);
			}
			selectedButton = cb;
			selectedButton.setSelected(true);
			colorDifference.setCurrentColor(cb.getBackground());
			colorDifference.setNewColor(cb.getBackground());
			cb.redraw();
		}
	}

	/**
	 * Returns the current selected colored button
	 * @return the current selected colored button
	 */
	public ColorButton getSelectedButton() {
		return selectedButton;
	}
}